﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;

namespace SerialPortSample3_CS
{
	public partial class Form1 : Form
	{
		/// <summary>シリアルポート</summary>
		private SerialPort port;

		/// <summary>読み込みバッファ</summary>
		private byte[] readBuffer = new byte[1024];

		/// <summary>readBuffer に格納済みのサイズ</summary>
		private int readBufferLength;

		/// <summary>
		/// txtChat コントロールに文字列を追加するためのデリゲート
		/// </summary>
		/// <param name="mes"></param>
		private delegate void AppendMessageDelegate(string mes);

		/// <summary>
		/// コンストラクタです。
		/// </summary>
		public Form1()
		{
			InitializeComponent();
		}

		/// <summary>
		/// フォームロードイベントです。
		/// シリアルポートをオープンします。
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_Load(object sender, EventArgs e)
		{
			this.port = new SerialPort("COM1", 9600, Parity.None, 8, StopBits.One);
			this.port.DataReceived += new SerialDataReceivedEventHandler(SerialPort_DataReceived);
			this.port.Open();
			this.port.DtrEnable = true;
			this.port.RtsEnable = true;
		}

		/// <summary>
		/// フォームクローズイベントです。
		/// シリアルポートをクローズします。
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_FormClosed(object sender, FormClosedEventArgs e)
		{
			this.port.Close();
			this.port.Dispose();
		}

		/// <summary>
		/// 送信ボタンクリックイベントです。
		/// メッセージを送信します。
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnSend_Click(object sender, EventArgs e)
		{
			string mes = this.txtMessege.Text.Trim();
			if (mes == "")
			{
				return;
			}

			byte[] buf = Encoding.GetEncoding("Shift_JIS").GetBytes(mes);
			short size = (short)buf.Length;
			this.port.Write(BitConverter.GetBytes(size), 0, sizeof(short));
			this.port.Write(buf, 0, size);

			this.txtChat.AppendText("自分: " + mes + "\r\n");
			this.txtMessege.Text = "";
		}

		/// <summary>
		/// シリアルポート受信イベントです。
		/// シリアルポートからデータを読み込みます。
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
		{
			int len = this.port.Read(this.readBuffer, this.readBufferLength, this.readBuffer.Length - this.readBufferLength);
			if (len == 0)
			{
				return;
			}

			this.readBufferLength += len;
			if (this.readBufferLength < sizeof(short))
			{
				// まだ先頭のデータサイズ部分を読み込めていない
				return;
			}
			while (true)
			{
				short size = BitConverter.ToInt16(this.readBuffer, 0);
				if (this.readBufferLength < size + 2)
				{
					// データサイズ部分は読み込んでいるがデータ部分の一部しか読み込んでいない
					if (this.readBuffer.Length < size + 2)
					{
						// バッファサイズが足らないので広げる
						byte[] buf = new byte[size + 2];
						Array.Copy(this.readBuffer, buf, this.readBuffer.Length);
						this.readBuffer = buf;
					}
					return;
				}
				string mes = Encoding.GetEncoding("Shift_JIS").GetString(this.readBuffer, sizeof(short), size);
				Array.Copy(this.readBuffer, sizeof(short) + size, this.readBuffer, 0, this.readBufferLength - sizeof(short) - size);
				this.readBufferLength -= sizeof(short) + size;

				this.BeginInvoke(new AppendMessageDelegate(AppendMessage), new object[] { mes });
			}
		}

		/// <summary>
		/// txtChat に文字列を追加します。
		/// </summary>
		/// <param name="mes"></param>
		private void AppendMessage(string mes)
		{
			this.txtChat.AppendText("相手: " + mes + "\r\n");
		}
	}
}