﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Threading;

namespace WindowsFormsApplication1
{

    // 光電管タイプ [中間計測用、スタート用、ゴール用、不明]
    public enum e_UNIT_TYPE : uint { MIDDLE, START, GOAL, UNKNOW };
    // データ受信状態 [未受信、通過台数受信中、通過台数受信済み、通過タイム受信中、通過タイム受信済み、受信完了]
    public enum e_RECV_STATUS : uint { UNRECV, PASSNUM, PASSNUM_COMP, PASSTIME, PASSTIME_COMP, COMP };
    // 表示状態 [未表示、表示済み]
    public enum e_DISP_STATUS : uint { UNDISP, DISP_COMP };
    // 計測状態 [未計測、計測済み、リタイヤ]
    public enum e_MESURE_STATUS : uint { UNMESURE, MESURE_COMP, RETIRE };

    class Measurement
    {
        private static SerialPort port = null;
        //private Thread MeasureThread = new Thread(MeasureProcess);
        private static System.Windows.Forms.Timer tmrMesure;
        private static System.Windows.Forms.Timer tmrRecvTimeout;
        //private static bool __MeasureContinue = false;
        private static bool __CommWait = false;

        private const byte c_COMMAND_MAXSIZE = 0x20;        // 受信コマンドバッファサイズ
        private const byte c_AL_COMMAND_A = 0x41;           // ALコマンドのA
        private const byte c_AL_COMMAND_L = 0x4C;           // ALコマンドのL
        private const byte c_RL_COMMAND_R = 0x52;           // RLコマンドのR
        private const byte c_RL_COMMAND_L = 0x4C;           // RLコマンドのL
        private const byte c_RL_COMMAND_SIZE = 0x06;        // RLコマンドサイズ
        private const byte c_AT_COMMAND_A = 0x41;           // ATコマンドのA
        private const byte c_AT_COMMAND_T = 0x54;           // ATコマンドのT
        private const byte c_RT_COMMAND_R = 0x52;           // RTコマンドのR
        private const byte c_RT_COMMAND_T = 0x54;           // RTコマンドのT
        private const byte c_RT_COMMAND_SIZE = 0x0A;        // RTコマンドサイズ
        private const int c_PROCESS_INTERVAL = 100;         // 処理周期
        //private const int c_PROCESS_INTERVAL = 10000;         // 処理周期[デバッグ用]
        private const int c_TIMEOUT_INTERVAL = 5000;        // 通信タイムアウト
        //private const int c_TIMEOUT_INTERVAL = 60000;        // 通信タイムアウト[デバッグ用]
        
        struct STR_MEASURE
        {
            public int Unit_No;                             //光電管番号
            public e_UNIT_TYPE Unit_Type;                   //光電管タイプ
            public int Pass_Num;                            //通過台数
            public e_RECV_STATUS Recv_Status;               //受信状態
            public List<e_DISP_STATUS> Disp_Status;         //表示状態
            public List<e_MESURE_STATUS> Mesure_Status;     //計測状態
            public List<int> lMSec;                         //ミリ秒
            public List<int> lSec;                          //秒
        };
        private static STR_MEASURE[] measu;

        /// <summary>
        /// 計測を開始する
        /// </summary>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        public int MeasurementStart()
        {
            int ret = 0;
            try
            {
                ret = MeasurementInit();
                port = SerialPortOpen();
                if (port != null)
                {
                    tmrMesure = new System.Windows.Forms.Timer();
                    tmrMesure.Tick += new EventHandler(MeasureProcess);
                    tmrMesure.Interval = c_PROCESS_INTERVAL;
                    tmrMesure.Start();

                    tmrRecvTimeout = new System.Windows.Forms.Timer();
                    tmrRecvTimeout.Tick += new EventHandler(RecvTimeOut);
                    tmrRecvTimeout.Interval = c_TIMEOUT_INTERVAL;

                    //__MeasureContinue = true;
                    ////MeasureThread = new Thread(MeasureProcess);
                    //MeasureThread.Start();
                }

            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 計測を停止する
        /// </summary>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        public int MeasurementStop()
        {
            int ret = 0;
            try
            {
                //__MeasureContinue = false;
                tmrMesure.Stop();
                tmrMesure = null;
                //if (MeasureThread != null)
                //{
                //    MeasureThread.Join();
                //    MeasureThread = null;
                //}
                ret = SerialPortClose(port);
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 計測値をクリアする
        /// </summary>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        public int MeasurementClear()
        {
            int ret = 0;
            try
            {
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 光電管数を取得する
        /// </summary>
        /// <returns>光電管数</returns>
        public int GetUnitNumber()
        {
            int ret = 0;
            try
            {
                ret = measu.Count();
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 光電管タイプを取得する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <returns>光電管タイプ</returns>
        public e_UNIT_TYPE GetUnitType(int paraUnitNo)
        {
            e_UNIT_TYPE ret = e_UNIT_TYPE.UNKNOW;
            try
            {
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        ret = measu[i].Unit_Type;
                        break;
                    }
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 通過台数を取得する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <returns>通過台数 0以上:通過台数 -1:光電管不明</returns>
        public int GetPassNumber(int paraUnitNo)
        {
            int ret = -1;
            try
            {
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        ret = measu[i].Pass_Num;
                        break;
                    }
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 表示状態を取得する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <param name="paraPassNo">スタート順（０～）</param>
        /// <returns>表示状態</returns>
        public e_DISP_STATUS GetDispStatus(int paraUnitNo, int paraPassNo)
        {
            e_DISP_STATUS ret = e_DISP_STATUS.UNDISP;
            try
            {
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        if (measu[i].Disp_Status.Count > paraPassNo)
                        {
                            ret = measu[i].Disp_Status[paraPassNo];
                            break;
                        }
                    }
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 表示状態を設定する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <param name="paraPassNo">スタート順（０～）</param>
        /// <param name="paraSetValue">設定する表示状態</param>
        /// <returns>処理結果 0:正常 !0:異常</returns>
        public int SetDispStatus(int paraUnitNo, int paraPassNo, e_DISP_STATUS paraSetValue)
        {
            int ret = 0;
            try
            {
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        if (measu[i].Disp_Status.Count > paraPassNo)
                        {
                            measu[i].Disp_Status[paraPassNo] = paraSetValue;
                            break;
                        }
                    }
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 計測状態を取得する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <param name="paraPassNo">スタート順（０～）</param>
        /// <returns>計測状態</returns>
        public e_MESURE_STATUS GetMesureStatus(int paraUnitNo, int paraPassNo)
        {
            e_MESURE_STATUS ret = e_MESURE_STATUS.UNMESURE;
            try
            {
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        if (measu[i].Mesure_Status.Count > paraPassNo)
                        {
                            ret = measu[i].Mesure_Status[paraPassNo];
                            break;
                        }
                    }
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 通過時間を取得する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <param name="paraPassNo">何番目に通過した車か（０～）</param>
        /// <returns>通過時間</returns>
        public string GetTime(int paraUnitNo, int paraPassNo)
        {
            string ret = "";
            int s_sec, s_msec, sec, msec;
            int getPassNum = GetMeasureArrayIndex(paraUnitNo, paraPassNo);
            e_MESURE_STATUS me_start = e_MESURE_STATUS.UNMESURE;
            e_MESURE_STATUS me_pass = e_MESURE_STATUS.UNMESURE;
            s_sec = 0;
            s_msec = 0;
            sec = 0;
            msec = 0;
            try
            {
                //対象車の通過タイムを取得
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    //光電管番号一致
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        //通過台数チェック
                        if (measu[i].Mesure_Status.Count > getPassNum && getPassNum >= 0)
                        {
                            if (measu[i].Mesure_Status[getPassNum] == e_MESURE_STATUS.MESURE_COMP)
                            {
                                me_pass = e_MESURE_STATUS.MESURE_COMP;
                                sec = measu[i].lSec[getPassNum];
                                msec = measu[i].lMSec[getPassNum];
                                break;
                            }                            
                        }
                    }
                }

                //対象車のスタートタイムを取得
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    if (measu[i].Unit_Type == e_UNIT_TYPE.START)
                    {
                        //通過台数チェック
                        if (measu[i].Mesure_Status.Count > getPassNum && getPassNum >= 0)
                        {
                            if (measu[i].Mesure_Status[getPassNum] == e_MESURE_STATUS.MESURE_COMP)
                            {
                                me_start = e_MESURE_STATUS.MESURE_COMP;
                                s_sec = measu[i].lSec[getPassNum];
                                s_msec = measu[i].lMSec[getPassNum];
                                break;
                            }

                        }
                    }
                }

                //対象車は対象光電管を通過済みで計測値取得ずみのため通過タイムを計算する
                if (me_start == e_MESURE_STATUS.MESURE_COMP && me_pass == e_MESURE_STATUS.MESURE_COMP)
                {
                    int i_temp_msec, i_temp_sec;
                    int i_hh, i_mm, i_ss;
                    //ミリ秒の差を計算
                    if (msec < s_msec)
                    {
                        s_sec--;
                        msec += 1000;
                    }
                    i_temp_msec = msec - s_msec;
                    i_temp_sec = sec - s_sec;

                    if (i_temp_sec >= 0 && i_temp_msec >= 0)
                    {
                        i_hh = i_temp_sec / 3600;
                        i_mm = i_temp_sec % 3600;
                        i_ss = i_mm % 60;
                        i_mm = i_mm / 60;
                        ret = i_hh.ToString() + ":" + i_mm.ToString("00") + ":" + i_ss.ToString("00") + "." + i_temp_msec.ToString("000");
                    }
                }
                else if (me_pass == e_MESURE_STATUS.RETIRE)
                {
                    ret = "--:--:--.---";
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 通過台数とリタイヤを考慮した配列位置を取得する
        /// </summary>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <param name="paraPassNo">光電管を何番目に通過した車か（０～）</param>
        /// <returns>配列インデックス</returns>
        private static int GetMeasureArrayIndex(int paraUnitNo, int paraPassNo)
        {
            int ret = -1;
            int recNum = 0;     //計測値格納位置
            int recCount = 0;   //リタイヤ以外のカウント用

            try
            {
                //リタイヤした数をカウント
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    //光電管番号一致
                    if (measu[i].Unit_No == paraUnitNo)
                    {
                        for (int j = 0; j < measu[i].Mesure_Status.Count; j++)
                        {
                            if (measu[i].Mesure_Status[j] == e_MESURE_STATUS.UNMESURE
                                || measu[i].Mesure_Status[j] == e_MESURE_STATUS.MESURE_COMP)
                            {
                                if (recCount == paraPassNo)
                                {
                                    break;
                                }
                                recCount++;
                                recNum++;
                            }
                            //リタイヤの場合は保存位置を１つ進める
                            if (measu[i].Mesure_Status[j] == e_MESURE_STATUS.RETIRE)
                            {
                                recNum++;
                            }
                        }

                        // 異常処理
                        if (measu[i].Mesure_Status.Count <= recNum)
                        {
                            recNum = -1;
                        }
                        ret = recNum;
                    }
                }

            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 計測にあたっての初期化処理を行う
        /// 光電管の設定を初期設定ファイルから読み込みバッファを設定
        /// </summary>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        private int MeasurementInit()
        {
            int ret = 0;
            try
            {
                //光電管の分、配列を用意する
                //光電管情報を設定ファイルから読み込む
                //仮で固定値
                measu = new STR_MEASURE[2];

                measu[0].Unit_No = 0;
                measu[0].Unit_Type = e_UNIT_TYPE.START;
                measu[1].Unit_No = 1;
                measu[1].Unit_Type = e_UNIT_TYPE.GOAL;

                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    measu[i].Recv_Status = e_RECV_STATUS.UNRECV;
                    //measu[i].Disp_Status = e_DISP_STATUS.UNDISP;
                    measu[i].lSec = new List<int>();
                    measu[i].lMSec = new List<int>();
                    measu[i].Disp_Status = new List<e_DISP_STATUS>();
                    measu[i].Mesure_Status = new List<e_MESURE_STATUS>();
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// シリアルポートをオープンする
        /// </summary>
        /// <returns>シリアルポートオブジェクト</returns>
        private SerialPort SerialPortOpen()
        {
            try
            {
                //シリアルポートが開いているか確認
                // 開いていたら一旦クローズする
                if (port != null)
                {
                    if (port.IsOpen == true)
                    {
                        port.Close();
                    }
                    port = null;
                }

                //通信設定を初期設定ファイルから読み込む
                //この処理は保留
                port = new SerialPort("COM2", 9600, Parity.None, 8, StopBits.One);
                port.DataReceived += new SerialDataReceivedEventHandler(SerialPort_DataReceived);
                port.ReceivedBytesThreshold = 1;
                port.Open();
                port.DtrEnable = true;
                port.RtsEnable = true;
            }
            catch
            {
                if (port != null)
                {
                    if (port.IsOpen == true)
                    {
                        port.Close();
                    }
                    port = null;
                }
            }
            finally
            { }
            return port;
        }

        /// <summary>
        /// シリアルポートをクローズする
        /// </summary>
        /// <param name="port">シリアルポートオブジェクト</param>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        private int SerialPortClose(SerialPort port)
        {
            int ret = 0;
            try
            {
                if (port != null)
                {
                    if (port.IsOpen == true)
                    {
                        port.Close();
                        //受信待ち解除
                        __CommWait = false;
                    }
                }
            }
            catch
            {
                ret = 1;
            }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void SerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            int ret = 0;
            try
            {

                SerialPort port = (SerialPort)sender;
                byte[] buf = new byte[c_COMMAND_MAXSIZE];
                int len = port.Read(buf, 0, c_COMMAND_MAXSIZE);
                if (len == buf[0])
                {
                    // 受信データ数正常
                    //タイムアウト解除
                    tmrRecvTimeout.Stop();

                    //受信データ解析
                    ret = Recv_Command_Analysis(buf, len);

                    //受信待ち解除
                    __CommWait = false;
                }
                string s = Encoding.GetEncoding("Shift_JIS").GetString(buf, 0, len);
                Console.WriteLine(s);
            }
            catch
            { }
            finally
            { }
        }

        private static int Recv_Command_Analysis(byte[] paraByte, int paraRecvLen)
        {
            int ret = 0;
            try
            {
                int len = paraByte.Length;

                if (paraRecvLen >= 3)
                {
                    if (paraByte[1] == c_RL_COMMAND_R && paraByte[2] == c_RL_COMMAND_L && paraRecvLen == c_RL_COMMAND_SIZE)
                    {   // ALコマンドのアンサRLコマンド
                        for(int i = 0; i <= measu.GetUpperBound(0); i++)
                        {
                            //光電管番号一致
                            if (measu[i].Unit_No == paraByte[3])
                            {
                                measu[i].Pass_Num = paraByte[5];
                                measu[i].Pass_Num = (measu[i].Pass_Num << 8) | paraByte[4];
                                if (paraByte[4] == 0xFF && paraByte[5] == 0xFF)
                                {   //問い合わせた光電管番号の光電管は存在しない場合
                                    measu[i].Pass_Num = 0;
                                }

                                //スタートした時点で計測時間格納用バッファを用意する
                                if (measu[i].Unit_Type == e_UNIT_TYPE.START)
                                {
                                    int startPassNum = measu[i].Pass_Num;
                                    for (int j = 0; j <= measu.GetUpperBound(0); j++)
                                    {
                                        for (int k = measu[j].Mesure_Status.Count; k < startPassNum; k++)
                                        {
                                            //計測状態
                                            measu[j].Mesure_Status.Add(e_MESURE_STATUS.UNMESURE);
                                            //秒
                                            measu[j].lSec.Add(0);
                                            //ミリ秒の設定
                                            measu[j].lMSec.Add(0);
                                            //表示状態
                                            measu[j].Disp_Status.Add(e_DISP_STATUS.UNDISP);
                                        }
                                    }
                                }
                                break;
                            }
                        }
                    }
                    else if (paraByte[1] == c_RT_COMMAND_R && paraByte[2] == c_RT_COMMAND_T && paraRecvLen == c_RT_COMMAND_SIZE)
                    {   //ATコマンドのアンサRTコマンド
                        for (int i = 0; i <= measu.GetUpperBound(0); i++)
                        {
                            //光電管番号一致
                            if (measu[i].Unit_No == paraByte[3])
                            {
                                if (paraByte[4] == 0xFF && paraByte[5] == 0xFF)
                                {   //問い合わせた光電管番号の光電管は存在しない場合
                                    break;
                                }
                                if (paraByte[6] == 0xFF && paraByte[7] == 0xFF
                                    && paraByte[8] == 0xFF && paraByte[9] == 0xFF)
                                {   //問い合わせた台数目の車はまだ未通過
                                    break;
                                }
                                int passNo = paraByte[5];
                                passNo = (passNo << 8) |paraByte[4];
                                //リタイヤした数をカウント
                                int recNum = GetMeasureArrayIndex(paraByte[3],passNo);     //計測値格納位置
                                
                                //配列範囲チェック
                                if (measu[i].Mesure_Status.Count > recNum && recNum >= 0)
                                {
                                    //未計測
                                    if (measu[i].Mesure_Status[recNum] == e_MESURE_STATUS.UNMESURE)
                                    {
                                        //計測状態を計測済みに設定
                                        measu[i].Mesure_Status[recNum] = e_MESURE_STATUS.MESURE_COMP;
                                        //秒の設定
                                        int iTime = paraByte[9];
                                        iTime = (iTime << 8) | paraByte[8];
                                        measu[i].lSec[recNum] = iTime;
                                        //ミリ秒の設定
                                        iTime = paraByte[7];
                                        iTime = (iTime << 8) | paraByte[6];
                                        measu[i].lMSec[recNum] = iTime;
                                        //表示状態
                                        measu[i].Disp_Status[recNum] = e_DISP_STATUS.UNDISP;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch
            { }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="myObject"></param>
        /// <param name="myEventArgs"></param>
        private static void MeasureProcess(Object myObject,EventArgs myEventArgs)
        {
            int ret = 0;
            try
            {
                //通信ポートオープン済み
                // 且つ　受信待ち以外
                if (port.IsOpen == true && __CommWait == false)
                {
                    bool sendComand = false;
                    for (int i = 0; i <= measu.GetUpperBound(0); i++ )
                    {
                        if (measu[i].Recv_Status == e_RECV_STATUS.UNRECV)
                        {
                            // 未受信のときは通過台数を問い合せる
                            measu[i].Recv_Status = e_RECV_STATUS.PASSNUM;
                            sendComand = true;

                            __CommWait = true;

                            ret = AL_Command_Send(port, measu[i].Unit_No);

                            break;
                        }
                        else if (measu[i].Recv_Status == e_RECV_STATUS.PASSNUM
                                || measu[i].Recv_Status == e_RECV_STATUS.PASSNUM_COMP
                                || measu[i].Recv_Status == e_RECV_STATUS.PASSTIME)
                        {
                            // 通過台数問い合わせ済みのときは通過タイムを問い合わせる
                            if(measu[i].Pass_Num > 0)
                            {
                                //通過タイム未取得のものあり
                                int passNum = 0;
                                for (int j = 0; j < measu[i].Mesure_Status.Count; j++)
                                {
                                    if (measu[i].Mesure_Status[j] == e_MESURE_STATUS.UNMESURE)
                                    {
                                        // 通過タイムの未受信データあり
                                        sendComand = true;
                                        measu[i].Recv_Status = e_RECV_STATUS.PASSTIME;

                                        __CommWait = true;

                                        ret = AT_Command_Send(port, measu[i].Unit_No, passNum);

                                        break;
                                    }
                                    if (measu[i].Mesure_Status[j] != e_MESURE_STATUS.RETIRE)
                                    {
                                        passNum++;
                                    }
                                }
                                if (sendComand == true)
                                {
                                    break;
                                }
                            }
                            measu[i].Recv_Status = e_RECV_STATUS.PASSTIME_COMP;
                        }
                        else if (measu[i].Recv_Status == e_RECV_STATUS.PASSTIME_COMP)
                        {
                            measu[i].Recv_Status = e_RECV_STATUS.COMP;
                        }

                    }
                    if (sendComand == false)
                    {
                        //問い合わせがない時は、受信状態を未受信に変更
                        for (int i = 0; i <= measu.GetUpperBound(0); i++)
                        {
                            measu[i].Recv_Status = e_RECV_STATUS.UNRECV;
                        }
                    }
                    else
                    {
                        //問い合わせありのときは受信待ちに設定
                        __CommWait = true;
                    }
                }
                DateTime dt = new DateTime();
                string message = dt.ToLongTimeString();
                //string message = port.ReadLine();
                Console.WriteLine(message);
            }
            catch (TimeoutException) { }
            //while (true)
            //{
            //    try
            //    {
            //        string message = "test";
            //        //string message = port.ReadLine();
            //        Console.WriteLine(message);
            //    }
            //    catch (TimeoutException) { }
            //}
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="myObject"></param>
        /// <param name="myEventArgs"></param>
        private static void RecvTimeOut(Object myObject, EventArgs myEventArgs)
        {
            try
            {
                tmrRecvTimeout.Stop();

                //受信状態を未受信に変更
                for (int i = 0; i <= measu.GetUpperBound(0); i++)
                {
                    measu[i].Recv_Status = e_RECV_STATUS.UNRECV;
                }

                __CommWait = false;

            }
            catch (TimeoutException) { }
        }

        /// <summary>
        /// ALコマンドを送信する
        /// </summary>
        /// <param name="port">シリアルポートオブジェクト</param>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        private static int AL_Command_Send(SerialPort port, int paraUnitNo)
        {
            int ret = 0;
            try
            {
                if (port != null)
                {
                    if (port.IsOpen == true)
                    {
                        // 送信前にバッファを読みすて
                        if (port.BytesToRead > 0)
                        {
                            byte[] tbuf = new byte[port.BytesToRead];
                            int len = port.Read(tbuf, 0, port.BytesToRead);
                        }

                        byte[] buf = new byte[4];
                        buf[0] = 0x04;
                        buf[1] = c_AL_COMMAND_A;
                        buf[2] = c_AL_COMMAND_L;
                        buf[3] = (byte)paraUnitNo;
                        port.Write(buf, 0, buf.Length);

                        tmrRecvTimeout.Start();
                    }
                }
            }
            catch
            {
                ret = 1;
            }
            finally
            { }
            return ret;
        }

        /// <summary>
        /// ATコマンドを送信する
        /// </summary>
        /// <param name="port">シリアルポートオブジェクト</param>
        /// <param name="paraUnitNo">光電管番号</param>
        /// <param name="paraPassNo">何台目に通過した車のタイムか</param>
        /// <returns>処理結果 0:正常 0以外:異常</returns>
        private static int AT_Command_Send(SerialPort port, int paraUnitNo, int paraPassNo)
        {
            int ret = 0;
            try
            {
                if (port != null)
                {
                    if (port.IsOpen == true)
                    {
                        // 送信前にバッファを読みすて
                        if (port.BytesToRead > 0)
                        {
                            byte[] tbuf = new byte[port.BytesToRead];
                            int len = port.Read(tbuf, 0, port.BytesToRead);
                        }

                        byte[] buf = new byte[6];
                        buf[0] = 0x06;
                        buf[1] = c_AT_COMMAND_A;
                        buf[2] = c_AT_COMMAND_T;
                        buf[3] = (byte)paraUnitNo;
                        buf[4] = (byte)(paraPassNo & 0xFF);
                        buf[5] = (byte)((paraPassNo >> 8) & 0xFF);
                        port.Write(buf, 0, buf.Length);

                        tmrRecvTimeout.Start();
                    }
                }
            }
            catch
            {
                ret = 1;
            }
            finally
            { }
            return ret;
        }
    }
}
