﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualBasic.FileIO;

namespace WindowsFormsApplication1
{
    class Register
    {
        /// <summary>
        /// 選手データファイルを読み込む
        /// </summary>
        /// <param name="paraFileName">選手データファイルパス</param>
        /// <param name="paraObj">選手データを表示するデータグリッドビュー</param>
        /// <returns>読み込み結果 0:正常 !0:異常</returns>
        public int DriverFile_Read(string paraFileName, Object paraObj)
        {
            try
            {
                TextFieldParser parser = new TextFieldParser(paraFileName, System.Text.Encoding.GetEncoding("Shift_JIS"));

                using (parser)
                {
                    DataGridView dgv = (DataGridView)paraObj;
                    dgv.Rows.Clear();
                    
                    parser.TextFieldType = FieldType.Delimited;
                    parser.SetDelimiters(","); // 区切り文字はコンマ

                    // parser.HasFieldsEnclosedInQuotes = false;
                    // parser.TrimWhiteSpace = false;

                    while (!parser.EndOfData)
                    {
                        string[] row = parser.ReadFields(); // 1行読み込み

                        dgv.Rows.Add(row);
                        /*
                        foreach (string field in row)
                        {
                            string f = field;
                            //dgv.Rows.Add(f);
                            f = f.Replace("\r\n", "n"); // 改行をnで表示
                            f = f.Replace(" ", "_"); // 空白を_で表示
                            Console.Write(f + "\t"); // TAB区切りで出力
                        }
                        Console.WriteLine();
                        */
                    }
                }
            }
            catch
            {}
            finally
            {}
            return 0;
        }

        /// <summary>
        /// 選手データをファイルに保存
        /// </summary>
        /// <param name="paraFileName">選手データファイルパス</param>
        /// <param name="paraObj">選手データが表示されているデータグリッドビュー</param>
        /// <returns>保存結果 0:正常 !0:異常</returns>
        public int DriverFile_Save(string paraFileName, Object paraObj)
        {
            try
            {
                // DataGridView
                DataGridView dgv = (DataGridView)paraObj;
                
                //CSVファイルに書き込むときに使うEncoding
                System.Text.Encoding enc = System.Text.Encoding.GetEncoding("Shift_JIS");

                //開く
                System.IO.StreamWriter sr = new System.IO.StreamWriter(paraFileName, false, enc);

                int colCount = dgv.Columns.Count;
                int lastColIndex = colCount - 1;
                int rowCount = dgv.Rows.Count;

                //レコードを書き込む
                for (int i = 0; i < rowCount; i++)
                {
                    for (int j = 0; j < colCount; j++)
                    {
                        //フィールドの取得
                        if (dgv[j, i].Value == null)
                        {
                            break;
                        }
                        string field = dgv[j,i].Value.ToString();
                        //"で囲む必要があるか調べる
                        /*if (field.IndexOf('"') > -1 ||
                            field.IndexOf(',') > -1 ||
                            field.IndexOf('\r') > -1 ||
                            field.IndexOf('\n') > -1 ||
                            field.StartsWith(" ") || field.StartsWith("\t") ||
                            field.EndsWith(" ") || field.EndsWith("\t"))
                        {*/
                            if (field.IndexOf('"') > -1)
                            {
                                //"を""とする
                                field = field.Replace("\"", "\"\"");
                            }
                            field = "\"" + field + "\"";
                        /*}*/
                        //フィールドを書き込む
                        sr.Write(field);
                        //カンマを書き込む
                        if (lastColIndex > i)
                        {
                            sr.Write(',');
                        }
                    }
                    //改行する
                    sr.Write("\r\n");
                }

                //閉じる
                sr.Close();

            }
            catch
            {}
            finally
            {}
            return 0; 
        }

        /// <summary>
        /// 選手データを表示しているDataGridViewから計測時間を表示しているDataGridViewに内容をコピーする
        /// </summary>
        /// <param name="paraObj1">計測時間のデータグリッドビューコントロール</param>
        /// <param name="paraObj2">選手データのデータグリッドビューコントロール</param>
        /// <returns>結果 0:正常 !0:異常</returns>
        public int DriverDataCopy(Object paraObj1, Object paraObj2)
        {
            try
            {
                // DataGridView
                DataGridView dgvTime = (DataGridView)paraObj1;
                dgvTime.Rows.Clear();
                // DataGridView
                DataGridView dgvDriver = (DataGridView)paraObj2;

                int colCountDr = dgvDriver.Columns.Count;
                int colCountTime = dgvDriver.Columns.Count;
                string[] row = new string[colCountDr]; // 1行読み込み
                int rowCount = dgvDriver.Rows.Count;
                DataGridViewColumn sortColumn = dgvTime.Columns[0];

                row[0] = "リタイヤ";
                //データをコピーする
                for (int i = 0; i < rowCount; i++)
                {
                    bool bFlag = false;
                    for (int j = 0; j < colCountDr; j++)
                    {
                        if (dgvDriver[j, i].OwningColumn.Name == "ColumnDr_CarNo")
                        {
                            for (int k = 0; k < colCountTime; k++)
                            {
                                if (dgvTime.Columns[k].Name == "ColumnTime_CarNo")
                                {
                                    sortColumn = dgvTime.Columns[k];
                                    if (dgvDriver[j, i].Value != null)
                                    {
                                        bFlag = true;
                                        row[k] = dgvDriver[j, i].Value.ToString();
                                    }
                                    break;
                                }
                            }
                        }
                        if (dgvDriver[j, i].OwningColumn.Name == "ColumnDr_MachineName")
                        {
                            for (int k = 0; k < colCountTime; k++)
                            {
                                if (dgvTime.Columns[k].Name == "ColumnTime_MachineName")
                                {
                                    if (dgvDriver[j, i].Value != null)
                                    {
                                        row[k] = dgvDriver[j, i].Value.ToString();
                                    }
                                    break;
                                }
                            }
                        }
                    }
                    if (bFlag == true)
                    {
                        dgvTime.Rows.Add(row);
                    }
                }
                //CarNoでソートする
                dgvTime.Sort(sortColumn, System.ComponentModel.ListSortDirection.Ascending);
                //Noを設定する
                int rowCountTime = dgvTime.Rows.Count;
                for (int i = 0; i < colCountTime; i++)
                {
                    if (dgvTime.Columns[i].Name == "ColumnTime_No")
                    {
                        for (int j = 0; j < rowCount; j++)
                        {
                            int no = j + 1;
                            dgvTime[i, j].Value = no.ToString();
                        }
                        break;
                    }
                }

            }
            catch
            {}
            finally
            {}
            return 0; 
        }

    }
}
